/*
 * Copyright 1991, 1992, John F. Haugh II
 * All rights reserved.
 *
 * Permission is granted to copy and create derivative works for any
 * non-commercial purpose, provided this copyright notice is preserved
 * in all copies of source code, or included in human readable form
 * and conspicuously displayed on all copies of object code or
 * distribution media.
 */

#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include "is_types.h"
#include "is_proto.h"
#include "isam.h"
#include "patchlevel.h"

#ifndef lint
static	char	sccsid[] = "@(#)dbshow.c	1.5	10:33:36	2/26/92";
#endif

dbm_t	*db_create();
dbm_t	*db_open();
int	db_open_file();
int	db_create_file();
int	db_create_field();

extern	int	db_errno;

/*
 * print_index - display the members of an index
 *
 * print_index() scans the list of key fields in an index and displays
 * the name for each of the fields.  It also looks at the ascending or
 * descending attribute and reports that information as well.
 */

print_index (file, index)
file_t	*file;
int	index;
{
	int	i, j;
	int	cnt;
	sort_key_t *keys;
	field_t	*fields,
		*field;

	/*
	 * Make certain the index is valid
	 */

	if (index <= 0 || index > file->f_index->i_key_count)
		return;

	/*
	 * Locate the index in the schema and determine how many
	 * fields are in it.
	 */

	keys = &file->f_index->i_keys[index];
	if (keys->k_unique)
		cnt = keys->k_field_count;
	else
		cnt = keys->k_field_count - 1;

	fields = file->f_schema->s_fields;

	/*
	 * For every field in the index, print out the name and if
	 * it is to be sorted in descending order, print out that
	 * information as well.
	 */

	for (i = 0;i < cnt;i++) {
		if (i > 0)
			printf (", ");
		else
			putchar ('\t');

		j = keys->k_fields[i] & ~ IS_DESCENDING;
		field = &fields[j];
		if (keys->k_fields[i] & IS_DESCENDING)
			printf ("%s descending", field->f_name);
		else
			printf ("%s", field->f_name);
	}
}

/*
 * dbshow - display a database schema
 *
 * dbshow displays the data dictionary for an entire database or a single
 * named file.
 */

main (argc, argv)
int	argc;
char	**argv;
{
	dbm_t	*dbm;
	file_t	*file;
	field_t	*fields;
	int	filenum;
	char	buf[BUFSIZ];
	field_t	new_field;
	int	field;
	int	i, j, k;
	int	verbose = 0;
	extern	int	optind;
	extern	char	*optarg;

	setbuf (stdout, 0);

	/*
	 * Command line options -
	 *
	 *	-V - show version number and exit
	 *	-v - show system fields as well
	 */

	while ((i = getopt (argc, argv, "Vv")) != EOF) {
		switch (i) {
			case 'V':
				fprintf (stderr, "dbshow: version %s\n",
					VERSION);
				exit (0);
			case 'v':
				verbose = 1;
				break;
		}
	}

	/*
	 * The file name is optional, but they have to give the database
	 * name.  If a single file is named it will be searched for in
	 * database and only the schema for that file will be displayed.
	 */

	if (optind >= argc || optind + 2 < argc) {
		fprintf (stderr, "usage: dbshow [ -Vv ] database [ file ]\n");
		exit (1);
	}
	if (! (dbm = db_open (argv[optind], O_RDONLY))) {
		sprintf (buf, "dbshow: can't open database %s", argv[optind]);
		db_error (buf);
		exit (1);
	}

	/*
	 * Skip over the database name and start printing out database files
	 */

	k = ++optind;

	if (dbm->d_file_count != 0) {

		/*
		 * Start with the name of the database, then work on each
		 * file entry
		 */

		printf ("database %s\n", dbm->d_dictionary.f_file_name);
		if (verbose)
			printf ("\n{ block size = %d }\n",
					dbm->d_dictionary.f_bsize);

		for (i = 0;i < dbm->d_file_count;i++) {

			/*
			 * Get a pointer to the next file entry, skipping
			 * the blank ones
			 */

			file = &dbm->d_files[i];

			if (! file->f_file_name[0])
				continue;

			/*
			 * See if this is the file the user asked to
			 * have displayed.  If they asked for one and
			 * this isn't it, skip it.
			 */

			if (optind != argc) {
				for (k = optind;k < argc;k++)
					if (! strcmp (argv[k],
							file->f_file_name))
						break;

				if (k == argc)
					continue;
			}

			/*
			 * Open the file to load up the schema into the
			 * dictionary.
			 */

			if (db_open_file (dbm, file->f_file_name) != i) {
				sprintf (buf, "dbshow: can't open file %s",
						file->f_file_name);
				db_error (buf);
				continue;
			}

			/*
			 * Display the name of the file and then go
			 * loop on the fields in the schema
			 */

			printf ("\nfile %s\n\n", file->f_file_name);

			if (verbose)
				printf ("{ record size = %d, block size = %d }\n\n",
					file->f_record_size, file->f_bsize);

			if (file->f_schema->s_fields == 0)
				continue;

			fields = file->f_schema->s_fields;

			for (j = 1;j < file->f_schema->s_field_count;j++) {

				/*
				 * Skip any system defined fields.  The
				 * user doesn't have to know about them.
				 */

				if (! verbose && fields[j].f_name[0] == '.')
					continue;

				/*
				 * Print out the field name and type.  Index
				 * fields take a little extra work.
				 */

				printf ("field\t%s\ttype", fields[j].f_name);
				switch (fields[j].f_type) {
					case T_CHAR:
						printf ("\tchar length %d\n",
							fields[j].f_size);
						break;
					case T_SHORT:
						printf ("\tshort\n");
						break;
					case T_INT:
						printf ("\tint\n");
						break;
					case T_LONG:
						printf ("\tlong\n");
						break;
					case T_DATE:
						printf ("\tdate\n");
						break;
					case T_SERIAL:
						printf ("\tserial\n");
						break;
					case T_FLOAT:
						printf ("\tfloat\n");
						break;
					case T_DOUBLE:
						printf ("\tdouble\n");
						break;
					case T_MONEY:
						printf ("\tmoney\n");
						break;
					case T_INDEX:
						printf ("\tindex");
						if (fields[j].f_size >= 0)
							printf (" unique");

						print_index (file,
							fields[j].f_offset);

						putchar ('\n');
						break;
				}
			}
		}
		printf ("\nend\n");
	}

	/*
	 * Close the open database.  Shouldn't be a big deal, but we
	 * check the error code in case something strange happens.
	 */

	if (db_close (dbm))
		db_error ("dbshow: can't close database");

	exit (0);
}
